@@ -21,6 +21,7 @@
 
 #include <QObject>
 #include <QSettings>
+#include <QTranslator>
 #include <QPixmap>
 
 class Configuration : public QObject
@@ -37,16 +38,20 @@ public:
 
     QString getLastLoginUser();
     void saveLastLoginUser(const QString &);
+    void saveLastLoginUser1(const QString &);
     bool getLastNumLock();
     void saveLastNumLock(bool value);
     QPixmap getLogo();
     QString getDefaultBackgroundName();
+    bool getUseFirstDevice();
+    QTranslator             *m_trans;
 
 private:
     QSettings               *configSettings;
     QSettings               *recordSettings;
 
     static Configuration    *instance_;
+
 };
 
 #endif // CONFIGURATION_H
