@@ -1,4 +1,4 @@
-﻿/* greeterwindow.cpp
+/* greeterwindow.cpp
  * Copyright (C) 2018 Tianjin KYLIN Information Technology Co., Ltd.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -19,12 +19,17 @@
 #include "greeterwindow.h"
 #include <QMenu>
 #include <QDebug>
+#include <QApplication>
 #include <QProcess>
+#include <QLayout>
+#include <QTranslator>
 #include <QStandardPaths>
 #include <QPainter>
+#include <QScrollBar>
 #include <QResizeEvent>
 #include <QWindow>
 #include <QtDBus/QDBusInterface>
+#include <QtConcurrent/QtConcurrentRun>
 #include <QtDBus/QDBusConnection>
 #include <QtDBus/QDBusReply>
 #include <QLightDM/SessionsModel>
@@ -39,6 +44,12 @@
 #include "languagewidget.h"
 #include "language.h"
 
+#include <fcntl.h>
+#include <unistd.h>
+#include <sys/types.h>
+#include <pwd.h>
+#include <libintl.h>
+
 float scale;
 int fontSize;
 
@@ -50,16 +61,13 @@ GreeterWindow::GreeterWindow(QWidget *parent)
       m_powerWnd(nullptr),
       m_sessionLB(nullptr),
       m_virtualKeyboard(nullptr),
-      m_languageWnd(nullptr),
       m_greeter(new GreeterWrapper()),
       m_usersModel(new UsersModel(m_greeter->hideUsersHint())),
       m_sessionsModel(new QLightDM::SessionsModel(QLightDM::SessionsModel::LocalSessions)),
       m_configuration(Configuration::instance()),
-      m_languageHasChanged(false),
       m_sessionHasChanged(false)
 {
     scale = 1.0;
-
     if(m_greeter->hasGuestAccountHint()){    //允许游客登录
         qDebug() << "allow guest";
         m_usersModel->setShowGuest(true);
@@ -71,78 +79,105 @@ GreeterWindow::GreeterWindow(QWidget *parent)
 
     m_greeter->setSession(m_greeter->defaultSessionHint());
 
-    for(int i = 0; i < m_usersModel->rowCount(); i++)
-        qDebug() << "load user " << m_usersModel->index(i).
-                    data(QLightDM::UsersModel::NameRole).toString();
-
-    for(int i = 0; i < m_sessionsModel->rowCount(); i++)
-        qDebug() << "load session " << m_sessionsModel->index(i, 0).
-                    data(QLightDM::SessionsModel::KeyRole).toString();
-
     connect(m_greeter, SIGNAL(autologinTimerExpired()),this, SLOT(timedAutologin()));
-    connect(m_greeter, SIGNAL(authenticationSucess()), this, SLOT(hide()));
-    connect(m_greeter, SIGNAL(startSessionFailed()), this, SLOT(show()));
+    connect(m_greeter, SIGNAL(authenticationComplete()),this, SLOT(onAuthenticationComplete1()));
     installEventFilter(this);
 }
 
+bool GreeterWindow::eventFilter(QObject *obj, QEvent *event)
+{
+    if(event->type() == QEvent::MouseButtonPress && obj == this){
+        if(m_powerWnd && m_powerWnd->isVisible()){
+            m_powerWnd->close();
+            switchWnd(1);
+        }
+    }
+    return false;
+}
+
 void GreeterWindow::initUI()
 {
     installEventFilter(this);
 
-    timer = new QTimer(this);
-    connect(timer, &QTimer::timeout, this, [&]{
-            QString time = QDateTime::currentDateTime().toString("hh:mm");
-            lblTime->setText(time);
-            QString date = QDate::currentDate().toString("yyyy/MM/dd dddd");
-            lblDate->setText(date);
-    });
-
     widgetTime = new QWidget(this);
     QVBoxLayout *widgetlayout = new QVBoxLayout(widgetTime);
     lblDate = new QLabel(this);
     lblTime = new QLabel(this);
-    widgetlayout->addWidget(lblTime);
-    widgetlayout->addWidget(lblDate);
-
-    QString time = QDateTime::currentDateTime().toString("hh:mm");
-    lblTime->setText(time);
-    lblTime->setStyleSheet("QLabel{color:white; font-size: 50px;}");
-    lblTime->adjustSize();
-    timer->start(1000);
-
-    QString date = QDate::currentDate().toString("yyyy/MM/dd dddd");
-    qDebug() << "current date: " << date;
-    lblDate->setText(date);
-    lblDate->setStyleSheet("QLabel{color:white; font-size: 16px;}");
-    lblDate->adjustSize();
-
-    widgetTime->adjustSize();
-
     //电源对话框打开按钮
     m_powerLB = new QPushButton(this);
-    m_powerLB->setObjectName(QStringLiteral("powerButton"));
-    m_powerLB->setIcon(QPixmap(":/images/power.png"));
-    m_powerLB->setIconSize(QSize(39, 39));
-    m_powerLB->setFocusPolicy(Qt::NoFocus);
-    m_powerLB->setFixedSize(48, 48);
-    m_powerLB->setCursor(Qt::PointingHandCursor);
-    m_powerLB->installEventFilter(this);
-    connect(m_powerLB, &QPushButton::clicked, this, &GreeterWindow::showPowerWnd);
 
     //虚拟键盘启动按钮
     m_keyboardLB = new QPushButton(this);
-    m_keyboardLB->setObjectName(QStringLiteral("keyboardButton"));
-    m_keyboardLB->setIcon(QPixmap(":/images/keyboard.png"));
-    m_keyboardLB->setIconSize(QSize(30, 30));
-    m_keyboardLB->setFocusPolicy(Qt::NoFocus);
-    m_keyboardLB->setFixedSize(48, 48);
-    m_keyboardLB->setCursor(Qt::PointingHandCursor);
-    m_keyboardLB->installEventFilter(this);
-    connect(m_keyboardLB, &QPushButton::clicked,
-            this, &GreeterWindow::showVirtualKeyboard);
+
+    //切换用户按钮
+    m_btnSwitchUser = new QPushButton(this);
+
+    timer = new QTimer(this);
+    connect(timer, &QTimer::timeout, this, [&]{
+                lblTime->setText(local.toString(QDateTime::currentDateTime(),"hh:mm"));
+                lblDate->setText(local.toString(QDate::currentDate(),"yyyy/MM/dd ddd"));
+    });
+    timer->start(1000);
+    QtConcurrent::run([=](){
+        local = QLocale::system().language();
+        QDateTime dateTime = QDateTime::currentDateTime();
+        QString strFormat = "dd.MM.yyyy, ddd MMMM d yy, hh:mm:ss.zzz, h:m:s ap";
+        QString strDateTime = local.toString(dateTime, strFormat);
+
+        widgetlayout->addWidget(lblTime);
+        widgetlayout->addWidget(lblDate);
+
+        lblTime->setText(local.toString(QDateTime::currentDateTime(),"hh:mm"));
+        lblTime->setStyleSheet("QLabel{color:white; font-size: 50px;}");
+
+        lblDate->setText(local.toString(QDate::currentDate(),"yyyy/MM/dd ddd"));
+        lblDate->setStyleSheet("QLabel{color:white; font-size: 16px;}");
+
+        widgetTime->adjustSize();
+
+        m_powerLB->setObjectName(QStringLiteral("powerButton"));
+        m_powerLB->setIcon(QPixmap(":/images/power.png"));
+        m_powerLB->setIconSize(QSize(30, 30));
+        m_powerLB->setFocusPolicy(Qt::NoFocus);
+        m_powerLB->setFixedSize(48, 48);
+        m_powerLB->setCursor(Qt::PointingHandCursor);
+        m_powerLB->installEventFilter(this);
+        connect(m_powerLB, &QPushButton::clicked, this, &GreeterWindow::showPowerWnd);
+
+        m_keyboardLB->setObjectName(QStringLiteral("keyboardButton"));
+        m_keyboardLB->setIcon(QPixmap(":/images/keyboard.png"));
+        m_keyboardLB->setIconSize(QSize(30, 30));
+        m_keyboardLB->setFocusPolicy(Qt::NoFocus);
+        m_keyboardLB->setFixedSize(48, 48);
+        m_keyboardLB->setCursor(Qt::PointingHandCursor);
+        m_keyboardLB->installEventFilter(this);
+        if(m_usersModel->rowCount() < 2){
+            m_btnSwitchUser->hide();
+        }
+	connect(m_keyboardLB, &QPushButton::clicked,
+                this, &GreeterWindow::showVirtualKeyboard);
+
+        m_btnSwitchUser->setObjectName(QStringLiteral("btnSwitchUser"));
+        m_btnSwitchUser->setIcon(QIcon(":/images/switchUser.png"));
+        m_btnSwitchUser->setIconSize(QSize(24, 24));
+        m_btnSwitchUser->setFocusPolicy(Qt::NoFocus);
+        m_btnSwitchUser->setFixedSize(48, 48);
+        m_btnSwitchUser->setCursor(Qt::PointingHandCursor);
+        m_btnSwitchUser->installEventFilter(this);
+        connect(m_btnSwitchUser, &QPushButton::clicked,
+                this, [&]{
+            if(m_powerWnd && m_powerWnd->isVisible())
+                m_powerWnd->close();
+
+            if(scrollArea->isVisible())
+                switchWnd(1);
+            else
+                switchWnd(0);
+        });
+    });
 
     //桌面环境选择按钮
-    if(m_sessionsModel->rowCount() > 0)
+    if(m_sessionsModel->rowCount() > 1)
     {
         m_sessionLB = new QPushButton(this);
         m_sessionLB->setObjectName((QStringLiteral("sessionButton")));
@@ -152,102 +187,152 @@ void GreeterWindow::initUI()
         m_sessionLB->setCursor(Qt::PointingHandCursor);
         m_sessionLB->installEventFilter(this);
         m_sessionLB->setIcon(QIcon(IMAGE_DIR + QString("badges/unknown_badge.svg")));
-        onSessionChanged(m_greeter->defaultSessionHint());
         connect(m_sessionLB, &QPushButton::clicked, this, &GreeterWindow::showSessionWnd);
     }
 
-    //语言选择按钮
-    m_languageLB = new QPushButton(this);
-    m_languageLB->setObjectName(QStringLiteral("languageButton"));
-    m_languageLB->setFocusPolicy(Qt::NoFocus);
-    m_languageLB->setFont(QFont("Ubuntu", 16));
-    m_languageLB->setFixedHeight(48);
-    m_languageLB->setCursor(Qt::PointingHandCursor);
-    m_languageLB->installEventFilter(this);
-    LanguagesVector defaultLang = getLanguages();
-    if(defaultLang.count()>0){
-        m_languageLB->setText(defaultLang.at(0).name);
-        m_languageLB->adjustSize();
-    }
-    else{
-        m_languageLB->hide();
-    }
-
-    connect(m_languageLB, &QPushButton::clicked, this, &GreeterWindow::showLanguageWnd);
-
     //用户列表
-    m_userWnd = new UsersView(this);
+    scrollArea = new QScrollArea(this);
+    scrollArea->setAttribute(Qt::WA_TranslucentBackground);
+    scrollArea->viewport()->setAttribute(Qt::WA_TranslucentBackground);
+    scrollArea->setStyleSheet("QScrollArea {background-color:transparent;}");
+    scrollArea->viewport()->setStyleSheet("background-color:transparent;");
+    scrollArea->verticalScrollBar()->setProperty("drawScrollBarGroove", false);
+    scrollArea->verticalScrollBar()->setContextMenuPolicy(Qt::NoContextMenu);
+    scrollContents = new QWidget(scrollArea);
+    scrollArea->setWidget(scrollContents);
+
+    m_userWnd = new UsersView(scrollContents);
     connect(m_userWnd, &UsersView::currentUserChanged, this, &GreeterWindow::onCurrentUserChanged);
     connect(m_userWnd, &UsersView::userSelected, this, &GreeterWindow::onUserSelected);
+    scrollArea->hide();
 
     //登录窗口
     m_loginWnd = new LoginWindow(m_greeter, this);
+    m_loginWnd->hide();
     connect(m_loginWnd, &LoginWindow::userChangedByManual,
             this, &GreeterWindow::onUserChangedByManual);
-    connect(m_loginWnd, &LoginWindow::bioDeviceIsChoosed,
-            this, &GreeterWindow::setUserWindowVisible);
 
     connect(m_userWnd, &UsersView::userNotFound, m_loginWnd, &LoginWindow::setUserNotInView);
 
     m_userWnd->setModel(m_usersModel);
     setFocusProxy(m_loginWnd);
+    QtConcurrent::run([=](){
+        //显示lightdm传过来的被选中的用户且自动进入认证界面 -- SwitchToUser()
+        QString selectedUser = m_greeter->selectUserHint();
 
-    //显示lightdm传过来的被选中的用户且自动进入认证界面 -- SwitchToUser()
+        if(!selectedUser.isEmpty())
+        {
+            qDebug() << "SelectUserHint: " << selectedUser;
+            m_userWnd->setCurrentUser(selectedUser, true);
+        }
+        // SwitchToGuest()且自动进入认证界面
+        else if(m_greeter->selectGuestHint())
+        {
+            qDebug() << "SelectGuest";
+            m_userWnd->setCurrentUser("*guest", true);
+        }
+        //如果没有可显示出来的用户，则开启手动输入用户名登录
+        else if(m_usersModel->rowCount() == 0)
+        {
+            m_usersModel->setShowManualLogin(true);
+            QString userName = m_usersModel->index(0, 0).data(QLightDM::UsersModel::NameRole).toString();
+            m_userWnd->setCurrentUser(userName, true);
+        }
+        //如果只有一个用户，直接进入认证界面
+        else if(m_usersModel->rowCount() == 1)
+        {
+            QString userName = m_usersModel->index(0, 0).data(QLightDM::UsersModel::NameRole).toString();
+            m_userWnd->setCurrentUser(userName, true);
+        }
+        else
+        {
+            //选中上一次登录的用户，但不进入认证界面
+            QString lastLoginUser = Configuration::instance()->getLastLoginUser();
+            m_userWnd->setCurrentUser(lastLoginUser);
+        }
+        setWindowOpacity(0.5);
+    });
+
+}
+
+void GreeterWindow::setRootWindow()
+{
+    m_greeter->setrootWindow();
+}
+
+void GreeterWindow::switchWnd(int index)
+{
+    if(m_userWnd)
+        scrollArea->hide();
+    if(m_loginWnd)
+        m_loginWnd->hide();
+
+    repaint();
+
+    switch (index) {
+    case 0:
+        if(m_userWnd)
+            scrollArea->show();
+        break;
+    case 1:
+        if(m_loginWnd)
+            m_loginWnd->show();
+        break;
+    default:
+        break;
+    }
+}
+
+QString GreeterWindow::guessBackground()
+{
     QString selectedUser = m_greeter->selectUserHint();
+
     if(!selectedUser.isEmpty())
     {
-        qDebug() << "SelectUserHint: " << selectedUser;
-        m_userWnd->setCurrentUser(selectedUser, true);
+        return m_configuration->getDefaultBackgroundName();
     }
-    // SwitchToGuest()且自动进入认证界面
     else if(m_greeter->selectGuestHint())
     {
-        qDebug() << "SelectGuest";
-        m_userWnd->setCurrentUser("*guest", true);
+        return m_configuration->getDefaultBackgroundName();
     }
-    //如果没有可显示出来的用户，则开启手动输入用户名登录
     else if(m_usersModel->rowCount() == 0)
     {
-        m_usersModel->setShowManualLogin(true);
-        QString userName = m_usersModel->index(0, 0).data(QLightDM::UsersModel::NameRole).toString();
-        m_userWnd->setCurrentUser(userName, true);
-    }
-    //如果只有一个用户，直接进入认证界面
-    else if(m_usersModel->rowCount() == 1)
-    {
-        QString userName = m_usersModel->index(0, 0).data(QLightDM::UsersModel::NameRole).toString();
-        m_userWnd->setCurrentUser(userName, true);
+        return m_configuration->getDefaultBackgroundName();
     }
     else
     {
-        //选中上一次登录的用户，但不进入认证界面
-        QString lastLoginUser = Configuration::instance()->getLastLoginUser();
-        m_userWnd->setCurrentUser(lastLoginUser);
-    }
+        QString userName;
+        if(m_usersModel->rowCount() == 1)
+            userName = m_usersModel->index(0, 0).data(QLightDM::UsersModel::NameRole).toString();
+        else
+            userName = Configuration::instance()->getLastLoginUser();
 
-    setWindowOpacity(0.5);
+        struct passwd *user = NULL;
+        user = getpwnam(userName.toLatin1().data());
+
+        if(user){
+            QString   backgroundPath = getAccountBackground(user->pw_uid);
+            if(backgroundPath.isEmpty())
+                backgroundPath = m_configuration->getDefaultBackgroundName();
+
+            QString userConfigurePath = m_greeter->getEnsureShareDir(userName) + "/ukui-greeter.conf";
+            QFile backgroundFile(userConfigurePath);
+            if(backgroundFile.exists()){
+                QSettings settings(userConfigurePath,QSettings::IniFormat);
+                settings.beginGroup("greeter");
+                if(settings.contains("backgroundPath")){
+                    QString filepath = settings.value("backgroundPath").toString();
+                    if(!filepath.isEmpty() && isPicture(filepath)){
+                        return filepath;
+                    }
+                }
+            }
+	    return backgroundPath;
+        }
+        return m_configuration->getDefaultBackgroundName();
+    }
 }
 
-void GreeterWindow::setUserWindowVisible(bool visible)
-{
-	//对距离进行缩放
-    if(m_userWnd)
-        m_userWnd->setVisible(visible);
-    if(!visible){
-        QRect loginRect((width()-m_loginWnd->width())/2,
-                        widgetTime->y() + widgetTime->height() + 176*scale,
-                        m_loginWnd->width(),
-                        m_loginWnd->height());
-        m_loginWnd->setGeometry(loginRect);
-    }
-    else{
-        QRect loginRect((width()-m_loginWnd->width())/2,
-                        m_userWnd->y() + m_userWnd->height() + 46 *scale,
-                        m_loginWnd->width(),
-                        height() - (m_userWnd->y() + m_userWnd->height() + 46 *scale));
-        m_loginWnd->setGeometry(loginRect);
-    }
-}
 /**
  * @brief GreeterWindow::resizeEvent
  * @param event
@@ -268,25 +353,36 @@ void GreeterWindow::resizeEvent(QResizeEvent *event)
 //对距离进行缩放
     widgetTime->move((width()-widgetTime->geometry().width())/2, 59*scale); //距离顶部59*scale的距离
 
+    if(scrollArea){
+        QRect userRect(0.05*width(),
+                       0.2*height(),
+                       width()*0.9, height()*0.6);
+        scrollArea->setGeometry(userRect);
+    }
+
+    if(scrollContents){
+        scrollContents->setFixedWidth(scrollArea->width()-20);
+    }
+
     if(m_userWnd){
-        m_userWnd->resize(CENTER_ENTRY_WIDTH*9 - ENTRY_WIDTH*4 + 240*scale, CENTER_ENTRY_HEIGHT);
-        QRect userRect((width()-m_userWnd->width())/2,
-                       widgetTime->geometry().bottom() + 176*scale,
-                       m_userWnd->width(), m_userWnd->height());
-        m_userWnd->setGeometry(userRect);
+        m_userWnd->resize(m_userWnd->getSize());
+        if(m_userWnd->height() + 2*30*scale  < scrollArea->height()){
+            m_userWnd->move((scrollArea->width()-m_userWnd->width())/2,(scrollArea->height() - m_userWnd->height())/2);
+        }else{
+            m_userWnd->move((scrollArea->width()-m_userWnd->width())/2,30*scale);
+        }
+        scrollContents->setFixedHeight(m_userWnd->height() + m_userWnd->y() );
     }
+
     if(m_loginWnd){
-
-    QRect loginRect((width()-m_loginWnd->width())/2,
-                        m_userWnd->y() + m_userWnd->height() + 46 *scale,
-                        m_loginWnd->width(),
-                        height() - (m_userWnd->y() + m_userWnd->height() + 46 *scale));
-        m_loginWnd->setGeometry(loginRect);
+        //m_loginWnd->setFixedHeight(height());
+        m_loginWnd->setChildrenGeometry();
+        m_loginWnd->setGeometry((width()-m_loginWnd->geometry().width())/2,height()/3, \
+                                m_loginWnd->width(), height()*2/3);
     }
-
-    if(m_languageWnd)
-    {
-        m_languageWnd->move(m_languageLB->x(),m_languageLB->y()+m_languageWnd->height());
+    
+    if(m_powerWnd){
+    	m_powerWnd->move((width()-m_powerWnd->width())/2,(height() - m_powerWnd->height())/2);
     }
 
     //电源按钮位置,根据x，y的值进行计算，初始值为到右下角的距离
@@ -294,12 +390,22 @@ void GreeterWindow::resizeEvent(QResizeEvent *event)
     int x = 39;
     int y = 66;
 
-    x+=m_powerLB->width();
-    m_powerLB->move(this->width() - x,height() - y);
+    if(m_powerLB){
+        x+=m_powerLB->width();
+        m_powerLB->move(this->width() - x,height() - y);
+    }
 
     //虚拟键盘按钮位置
-    x += (m_keyboardLB->width() + 10); //10为间隔
-    m_keyboardLB->move(this->width() - x, height() - y);
+    if(m_keyboardLB){
+        x += (m_keyboardLB->width() + 10); //10为间隔
+        m_keyboardLB->move(this->width() - x, height() - y);
+    }
+
+    //切换用户按钮位置
+    if(m_btnSwitchUser){
+        x += (m_btnSwitchUser->width() + 10); //10为间隔
+        m_btnSwitchUser->move(this->width() - x, height() - y);
+    }
 
     //桌面环境选择按钮位置
     if(m_sessionLB)
@@ -308,10 +414,6 @@ void GreeterWindow::resizeEvent(QResizeEvent *event)
         m_sessionLB->move(this->width() - x, height() - y);
     }
 
-    //语言选择按钮位置
-    x += (m_languageLB->width() + 10); //10为间隔
-    m_languageLB->move(this->width() - x, height() - y);
-
     //虚拟键盘位置
     setVirkeyboardPos();
 
@@ -328,58 +430,23 @@ void GreeterWindow::setVirkeyboardPos()
     }
 }
 
-bool GreeterWindow::eventFilter(QObject *obj, QEvent *event)
-{
-    if(event->type() == QEvent::MouseButtonPress){
-        if(obj == m_sessionLB || obj == m_languageLB || obj == m_powerLB)
-            return false;
-
-        if(m_sessionWnd && m_sessionWnd->isVisible())
-        {
-            m_sessionWnd->close();
-            m_sessionWnd = nullptr;
-            update();
-        }
-        if(m_languageWnd && m_languageWnd->isVisible())
-        {
-            m_languageWnd->close();
-            m_languageWnd = nullptr;
-            update();
-        }
-        if(m_powerWnd && !m_powerWnd->isHidden()){
-            m_powerWnd->close();
-            m_userWnd->show();
-            m_loginWnd->show();
-            update();
-        }
-    }
-    return false;
-}
-
 void GreeterWindow::keyReleaseEvent(QKeyEvent *e)
 {
     switch(e->key())
     {
-    case Qt::Key_L:
-        if(e->modifiers() & Qt::ControlModifier)
-            m_languageLB->click();
-    break;
     case Qt::Key_K:
         if(e->modifiers() & Qt::ControlModifier)
             m_keyboardLB->click();
-    break;
+        break;
     case Qt::Key_P:
         if(e->modifiers() & Qt::ControlModifier)
             m_powerLB->click();
-    break;
+        break;
     case Qt::Key_Escape:
         if(m_powerWnd && !m_powerWnd->isHidden()){
-            m_powerWnd->close();
-            m_userWnd->show();
-            m_loginWnd->show();
-            update();
+            setWindowVisible();
         }
-    break;
+        break;
     }
     QWidget::keyReleaseEvent(e);
 }
@@ -406,9 +473,23 @@ QString GreeterWindow::getAccountBackground(uid_t uid)
 void GreeterWindow::setBackground(const QModelIndex &index)
 {
     QString backgroundPath;
+    QString accountBackground; //记录来自accountsservice的背景图片
+
+    backgroundPath = index.data(QLightDM::UsersModel::BackgroundPathRole).toString();
+    accountBackground = backgroundPath;
+
+    if(backgroundPath.isEmpty())
+    {
+        uid_t uid = index.data(QLightDM::UsersModel::UidRole).toUInt();
+        backgroundPath = getAccountBackground(uid);
+        accountBackground = backgroundPath;
+        if(backgroundPath.isEmpty())
+            backgroundPath = m_configuration->getDefaultBackgroundName();
+    }
+
+    //登录后绘制桌面背景而不是登录背景
+    m_greeter->setrootWindowBackground(rootWinPicture,0,backgroundPath);
 
-    bool useUserBackground = false;
-	
     //读取/var/lib/lightdm-date/用户名/ukui-greeter.conf,
     //判断是否设置了该用户的登陆界面的背景图片.
     QString userConfigurePath = m_greeter->getEnsureShareDir(index.data(QLightDM::UsersModel::NameRole).toString()) + "/ukui-greeter.conf";
@@ -420,27 +501,32 @@ void GreeterWindow::setBackground(const QModelIndex &index)
             QString filepath = settings.value("backgroundPath").toString();
             if(!filepath.isEmpty() && isPicture(filepath)){
                 backgroundPath = filepath;
-                useUserBackground = true;
+            }
+        }
+        if(accountBackground.isEmpty() && settings.contains("color")){
+            QString drawBackgroundColor = settings.value("color").toString();
+       	    if(drawBackgroundColor.length() == 7 && drawBackgroundColor.startsWith("#")){
+            drawBackgroundColor = drawBackgroundColor.remove(0,1);
+                bool ok;
+                int color = drawBackgroundColor.toUInt(&ok,16);
+                m_greeter->setrootWindowBackground(rootWinColor,color,NULL);
             }
         }
         settings.endGroup();
+    }else{
+    	QSettings settingsbak(userConfigurePath,QSettings::IniFormat);
+        settingsbak.beginGroup("greeter");
+        settingsbak.setValue("backgroundPath","/usr/share/backgrounds/warty-final-ubuntukylin.jpg");
+        settingsbak.endGroup();
+        settingsbak.sync();
+        QFile file(userConfigurePath);
+        file.setPermissions(QFile::WriteOther | QFile::ReadOther);
     }
 
-    if(!useUserBackground){
-        backgroundPath = index.data(QLightDM::UsersModel::BackgroundPathRole).toString();
-        if(backgroundPath.isEmpty())
-        {
-            uid_t uid = index.data(QLightDM::UsersModel::UidRole).toUInt();
-            backgroundPath = getAccountBackground(uid);
-            if(backgroundPath.isEmpty())
-                backgroundPath = m_configuration->getDefaultBackgroundName();
-        }
-    }
     QSharedPointer<Background> background(new Background);
     background->type = BACKGROUND_IMAGE;
     background->image = backgroundPath;
     static_cast<MainWindow*>(parentWidget())->setBackground(background);
-    m_greeter->setrootWindowBackground(background->image);
 
 }
 
@@ -449,11 +535,14 @@ void GreeterWindow::onUserSelected(const QModelIndex &index)
     qDebug() << index.data(QLightDM::UsersModel::NameRole).toString() << "selected";
     m_loginWnd->setUserIndex(index);
 
+    switchWnd(1);
+
 }
 
 void GreeterWindow::updateLanguage(QString userName)
 {
     QString language;
+    QString formatsLocale;
     QDBusInterface iface("org.freedesktop.Accounts", "/org/freedesktop/Accounts",
                          "org.freedesktop.Accounts",QDBusConnection::systemBus());
     QDBusReply<QDBusObjectPath> userPath = iface.call("FindUserByName", userName);
@@ -468,9 +557,29 @@ void GreeterWindow::updateLanguage(QString userName)
         else {
             language = languageReply.value().variant().toString();
             if(!language.isEmpty()){
-                    onLanguageChanged(getLanguage(language));
+                    onLanguageChanged(language);
              }
         }
+
+        QDBusReply<QDBusVariant> fomatsLocalReply = userIface.call("Get", "org.freedesktop.Accounts.User", "FormatsLocale");
+        if(!fomatsLocalReply.isValid())
+            qWarning() << "Get User's language error" << languageReply.error();
+        else {
+            formatsLocale = fomatsLocalReply.value().variant().toString();
+            if(!formatsLocale.isEmpty()){
+                if(formatsLocale.startsWith("zh")){
+                    local = QLocale::Chinese;
+            QString date = local.toString(QDate::currentDate(),"yyyy/MM/dd ddd");
+                    lblDate->setText(date);
+                }
+                else{
+                    local = QLocale::English;
+                    QString date = local.toString(QDate::currentDate(),"yyyy/MM/dd ddd");
+                    lblDate->setText(date);
+        }
+            }
+        }
+
     }
 }
 
@@ -496,23 +605,55 @@ void GreeterWindow::updateSession(QString userName)
 
 void GreeterWindow::onCurrentUserChanged(const QModelIndex &index)
 {
-    setBackground(index);
-
-    if(!m_languageHasChanged)
+    for(int i = 0;i<m_userWnd->userlist.count();i++)
     {
-        //获取用户的session语言
-        QString language;
-        QString realName = index.data(QLightDM::UsersModel::NameRole).toString();
-        if(realName == "*guest" || realName == "*login")
-            return;
-        updateLanguage(realName);
+        UserEntry *entry = m_userWnd->userlist.at(i);
+          if(entry->userIndex().data(QLightDM::UsersModel::NameRole).toString() == "*login")
+               entry->setUserName(tr("Login"));
     }
 
+    setBackground(index);
+
+
+    if(m_greeter->getdrawBackgroundIsStarted()){
+        if(future.isStarted()&& future.isRunning()){
+            future.waitForFinished();
+        }else{
+            future = QtConcurrent::run([=](){
+                m_greeter->setrootWindow();
+            });
+        }
+    }else{
+        future = QtConcurrent::run([=](){
+            m_greeter->setrootWindow();
+        });
+    }
+
+    QString realName = index.data(QLightDM::UsersModel::NameRole).toString();
+    bool islogin = index.data(QLightDM::UsersModel::LoggedInRole).toBool();
+
+    if(islogin || realName == "*login" || realName == "*guest"){
+        if(m_sessionLB){
+            m_sessionLB->hide();
+        }
+    }
+    else{
+        if(m_sessionLB){
+            m_sessionLB->show();
+        }
+    }
+
+    if(realName == "*guest" || realName == "*login")
+        return;
+
+    updateLanguage(realName);
     if(!m_sessionHasChanged && m_sessionsModel->rowCount() > 1)
     {
         QString session = index.data(QLightDM::UsersModel::SessionRole).toString();
         onSessionChanged(session);
     }
+
+    update();
 }
 
 void GreeterWindow::onUserChangedByManual(const QString &userName)
@@ -526,10 +667,9 @@ void GreeterWindow::onUserChangedByManual(const QString &userName)
 void GreeterWindow::setWindowVisible()
 {
     if(m_powerWnd && m_powerWnd->isVisible())
-        m_powerWnd->hide();
+        m_powerWnd->close();
 
-    m_loginWnd->setVisible(true);
-    m_userWnd->setVisible(true);
+    switchWnd(1);
     update();
 }
 /**
@@ -541,13 +681,12 @@ void GreeterWindow::showPowerWnd()
     //如果已经打开了电源对话框则关闭
     if(m_powerWnd && !m_powerWnd->isHidden()){
         m_powerWnd->close();
-        m_userWnd->show();
-        m_loginWnd->show();
+        switchWnd(1);
         update();
         return;
     }
 
-    m_userWnd->hide();
+    scrollArea->hide();
     m_loginWnd->hide();
 
     m_powerWnd = new PowerManager(this);
@@ -558,8 +697,16 @@ void GreeterWindow::showPowerWnd()
     m_powerWnd->setFixedSize(m_powerWnd->windowSize());
     m_powerWnd->move((width()-m_powerWnd->width())/2,(height() - m_powerWnd->height())/2);
 
-    connect(m_powerWnd,SIGNAL(switchToUser())
-            ,this,SLOT(setWindowVisible()));
+//    connect(m_powerWnd,SIGNAL(switchToUser())
+//            ,this,SLOT(setWindowVisible()));
+    connect(m_powerWnd, &PowerManager::switchToUser, this,
+            [this]() {
+        if(m_powerWnd && m_powerWnd->isVisible())
+            m_powerWnd->close();
+
+        switchWnd(0);
+    });
+
     m_powerWnd->setObjectName(QStringLiteral("powerWnd"));
     m_powerWnd->show();
     update();
@@ -599,33 +746,6 @@ void GreeterWindow::timedAutologin()
         m_greeter->authenticateAutologin();
 }
 
-/**
- * @brief GreeterWindow::showLanguageWnd
- * 显示语言选择窗口
- */
-void GreeterWindow::showLanguageWnd()
-{
-    if(!m_languageWnd){
-        m_languageWnd = new LanguageWidget(this);
-        m_languageWnd->setObjectName("languageWnd");
-
-        connect(m_languageWnd, &LanguageWidget::languageChanged,
-            this, &GreeterWindow::onLanguageChanged);
-    }
-    m_languageHasChanged = true;
-    if(m_languageWnd->isVisible()){
-        m_languageWnd->close();
-        m_languageWnd = nullptr;
-        setFocus();
-    }
-    else{
-        m_languageWnd->show();
-        m_languageWnd->setFocus();
-        m_languageWnd->move(m_languageLB->x(),m_languageLB->y()-m_languageWnd->height() - 3);
-    }
-    update();
-}
-
 void GreeterWindow::setWindowPos(QWidget *widget, Qt::Alignment align)
 {
     QRect rect;
@@ -646,48 +766,37 @@ void GreeterWindow::setWindowPos(QWidget *widget, Qt::Alignment align)
     widget->setGeometry(rect);
 }
 
-void GreeterWindow::onLanguageChanged(const Language &language)
+void GreeterWindow::onLanguageChanged(QString languageCode)
 {
-    if(language.code == "" || language.name == "")
+
+    if(languageCode == "")
         return ;
 
-    if(m_greeter->lang() == language.code)
+    if(m_greeter->lang() == languageCode)
     {
         return;
     }
 
-    m_greeter->setLang(language.code);
+    qApp->removeTranslator(m_configuration->m_trans);
+    delete m_configuration->m_trans;
+    m_configuration->m_trans = new QTranslator();
+    QString qmFile;
 
-    m_languageLB->setText(language.name);
+    if(languageCode.startsWith("zh")){
+        qmFile = QM_DIR + QString("%1.qm").arg("zh_CN");
+        setenv("LANGUAGE","zh_CN",1);
+        setlocale(LC_ALL,"zh_CN.utf8");
+    }
+    else{
+        qmFile = QM_DIR + QString("%1.qm").arg(languageCode);
+        setenv("LANGUAGE",languageCode.toLatin1().data(),1);
+        setlocale(LC_ALL,"");
+    }
+    m_configuration->m_trans->load(qmFile);
+    qApp->installTranslator(m_configuration->m_trans);
 
-    const QFont &font = m_languageLB->font();
-    QFontMetrics fm(font);
-    int pixelWidth = fm.width(language.name);
-//根据语言文字的长度计算按钮的宽度，并加上文字距离按钮两边的间距
-    m_languageLB->setFixedWidth(pixelWidth + 14);
+    m_greeter->setLang(languageCode);
 
-    int x,y;
-    if(m_sessionLB)
-    {
-        x = m_sessionLB->geometry().left();
-        y = m_sessionLB->y();
-    }
-    else
-    {
-        x = m_keyboardLB->geometry().left();
-        y = m_keyboardLB->y();
-    }
-//10为距离左边按钮的距离
-    m_languageLB->move(x - 10 - m_languageLB->width() , y);
-
-    if(m_userWnd && !m_userWnd->isHidden())
-    {
-        m_userWnd->setFocus();
-    }
-    if(m_languageWnd){
-        m_languageWnd->close();
-        m_languageWnd = nullptr;
-    }
 }
 
 void GreeterWindow::showSessionWnd()
@@ -713,7 +822,7 @@ void GreeterWindow::showSessionWnd()
         setFocus();
     }
     else{
-        m_sessionWnd->move(m_languageLB->x(),m_languageLB->y()-m_sessionWnd->height() - 3);
+        m_sessionWnd->move(m_sessionLB->x(),m_sessionLB->y()-m_sessionWnd->height() - 3);
         m_sessionWnd->show();
         m_sessionWnd->setFocus();
     }
@@ -744,8 +853,10 @@ void GreeterWindow::onSessionChanged(const QString &session)
             sessionTmp = "";
         }
     }
-
+	
     QString sessionPrefix = sessionTmp.left(sessionTmp.indexOf('-'));
+    if(sessionTmp == "ukui-wayland")
+	    sessionPrefix = "ukui_wayland";
     QString sessionIcon = IMAGE_DIR + QString("badges/%1_badge.svg")
             .arg(sessionPrefix.toLower());
     QFile iconFile(sessionIcon);
@@ -782,3 +893,12 @@ bool GreeterWindow::sessionIsValid(const QString &session)
     return false;
 }
 
+void GreeterWindow::onAuthenticationComplete1()
+{
+    for(int i = 0;i<m_userWnd->userlist.count();i++)
+    {
+        UserEntry *entry = m_userWnd->userlist.at(i);
+          if(entry->userIndex().data(QLightDM::UsersModel::NameRole).toString() == "*login")
+               entry->setUserName(tr("Login"));
+    }
+}
