@@ -25,7 +25,10 @@
 #include <QDebug>
 #include <QDir>
 #include <QScreen>
+#include <QWindow>
+#include <QtConcurrent/QtConcurrentRun>
 #include <QX11Info>
+#include <QByteArray>
 #include <QProcess>
 #include <QtMath>
 #include <QTimer>
@@ -34,11 +37,16 @@
 #include "common/configuration.h"
 #include "common/monitorwatcher.h"
 #include "display-switch/displayservice.h"
+#include <xcb/xcb.h>
 #include <X11/XKBlib.h>
 #include <X11/Xlib.h>
+#include <X11/extensions/Xrandr.h>
 #include <X11/keysymdef.h>
+#include <X11/cursorfont.h>
 #include <X11/keysym.h>
-
+#include <libintl.h>
+#include <unistd.h>
+#include "xeventmonitor.h"
 bool MainWindow::m_first = true;
 
 QT_BEGIN_NAMESPACE
@@ -61,30 +69,21 @@ MainWindow::MainWindow(QWidget *parent)
       m_screenModel(new ScreenModel(this)),
       m_configuration(Configuration::instance()),
       m_activeScreen(nullptr),
-      m_monitorWatcher(new MonitorWatcher(this)),
-      m_timer(nullptr),
-      m_background(nullptr)
+      m_background(nullptr),
+      m_monitorCount(0)
 {
+    setWindowFlags(Qt::X11BypassWindowManagerHint);
+    XRRQueryExtension(QX11Info::display(), &rr_event_base, &rr_error_base);
+    XRRSelectInput(QX11Info::display(), QX11Info::appRootWindow(), RRScreenChangeNotifyMask);
+
     QDesktopWidget *_desktop = QApplication::desktop();
     connect(_desktop, &QDesktopWidget::resized, this, &MainWindow::onScreenResized);
-    /* QDesktopWidget对显示器的插拔的支持不好 */
-    connect(m_monitorWatcher, &MonitorWatcher::monitorCountChanged, this, &MainWindow::onScreenCountChanged);
-    connect(_desktop, &QDesktopWidget::screenCountChanged, this, &MainWindow::onScreenCountChanged);
+    connect(_desktop, &QDesktopWidget::screenCountChanged, this, &MainWindow::onScreenResized);
     //设置窗口大小
-    int totalWidth = 0;
-    int totalHeight = 0;
-    for(auto screen : QGuiApplication::screens())
-    {
-        totalWidth += screen->geometry().width();
-        totalHeight += screen->geometry().height();
-    }
-    setGeometry(0, 0, totalWidth, totalHeight);
+    qDebug()<<_desktop->geometry();
+    setGeometry(_desktop->geometry());
     //设置监控鼠标移动
     setMouseTracking(true);
-
-    //cof
-    //m_cof.load(m_configuration->getValue("cof").toString());
-
     //背景图片 优先级：用户桌面背景、背景图片、背景颜色
     m_defaultBackgroundPath = m_configuration->getDefaultBackgroundName();
     bool drawUserBackground = m_configuration->getValue("draw-user-background").toBool();
@@ -113,35 +112,44 @@ MainWindow::MainWindow(QWidget *parent)
             }
         }
     }
-    qDebug() << "default background:" << m_defaultBackgroundPath;
-    if(m_background)
-        qDebug() << m_background->type << m_background->image;
 
-    m_timer = new QTimer(this);
     m_transition.started = false;
 
-    //激活屏幕(即Greeter窗口所在屏幕位置)
     m_greeterWnd = new GreeterWindow(this);
-    moveToScreen(QApplication::primaryScreen());
+    future = QtConcurrent::run([=](){
+        QString backgroundname = m_greeterWnd->guessBackground();
+        QString resolution = QString("%1x%2").arg(QApplication::primaryScreen()->geometry().width()).arg(QApplication::primaryScreen()->geometry().height());
+        QPair<QString, QString> key(backgroundname, resolution);
+        if(!m_backgrounds.contains(key)){
+            QPixmap *pixmap =  new QPixmap(scaledPixmap(QApplication::primaryScreen()->geometry().width(), QApplication::primaryScreen()->geometry().height(), backgroundname));
+            m_backgrounds[key] = blurPixmap(pixmap);
+        }
+    });
+
+    //登录界面移动到鼠标所在的屏幕上
+    if(QApplication::screens().count() > 1){
+        QPoint point = QCursor::pos();
+        QScreen *curScreen = nullptr;
+        for(QScreen *screen : QApplication::screens()){
+            QRect screenRect = screen->geometry();
+            if(screenRect.contains(point)) {
+                curScreen = screen;
+                break;
+            }
+        }
+        if(curScreen != m_activeScreen && curScreen != nullptr){
+            qDebug() << "active screen: from " << m_activeScreen << "to " << curScreen;
+            moveToScreen(curScreen);
+        }
+    }else{
+        moveToScreen(QApplication::primaryScreen());
+    }
+
     m_greeterWnd->initUI();
 
-    m_monitorWatcher->start();
+    qApp->installNativeEventFilter(this);
 
-    connect(m_timer, &QTimer::timeout, this, &MainWindow::onTransition);
-
-    bool numlockState = true;
-    numlockState = m_configuration->getLastNumLock();
-
-    if(numlockState){
-        //默认打开numlock需要设置两次，否则灯和效果可能不一致，原因不知
-        unsigned int num_mask = XkbKeysymToModifiers (QX11Info::display(), XK_Num_Lock);
-        XkbLockModifiers (QX11Info::display(), XkbUseCoreKbd, num_mask, 0);
-        XkbLockModifiers (QX11Info::display(), XkbUseCoreKbd, num_mask, num_mask);
-    }else{
-        unsigned int num_mask = XkbKeysymToModifiers (QX11Info::display(), XK_Num_Lock);
-        XkbLockModifiers (QX11Info::display(), XkbUseCoreKbd, num_mask, num_mask);
-        XkbLockModifiers (QX11Info::display(), XkbUseCoreKbd, num_mask, 0);
-    }
+    showLater();
 
 }
 
@@ -188,34 +196,91 @@ void MainWindow::mouseMoveEvent(QMouseEvent *e)
     return QWidget::mouseMoveEvent(e);
 }
 
+void MainWindow::showLater()
+{
+    if(checkNumLockState()){
+        unsigned int num_mask = XkbKeysymToModifiers (QX11Info::display(), XK_Num_Lock);
+        XkbLockModifiers (QX11Info::display(), XkbUseCoreKbd, num_mask, 0);
+        XkbLockModifiers (QX11Info::display(), XkbUseCoreKbd, num_mask, num_mask);
+    }else{
+        unsigned int num_mask = XkbKeysymToModifiers (QX11Info::display(), XK_Num_Lock);
+        XkbLockModifiers (QX11Info::display(), XkbUseCoreKbd, num_mask, num_mask);
+        XkbLockModifiers (QX11Info::display(), XkbUseCoreKbd, num_mask, 0);
+    }
+
+    if(checkCapsState()){
+        unsigned int caps_mask = XkbKeysymToModifiers (QX11Info::display(), XK_Caps_Lock);
+        XkbLockModifiers (QX11Info::display(), XkbUseCoreKbd, caps_mask, 0);
+        XkbLockModifiers (QX11Info::display(), XkbUseCoreKbd, caps_mask, caps_mask);
+    }else{
+        unsigned int caps_mask = XkbKeysymToModifiers (QX11Info::display(), XK_Caps_Lock);
+        XkbLockModifiers (QX11Info::display(), XkbUseCoreKbd, caps_mask, caps_mask);
+        XkbLockModifiers (QX11Info::display(), XkbUseCoreKbd, caps_mask, 0);
+    }
+
+    XDefineCursor(QX11Info::display(), QX11Info::appRootWindow(), XCreateFontCursor(QX11Info::display(), XC_arrow));
+
+    bindtextdomain("Linux-PAM","/usr/share/locale");
+    textdomain("Linux-PAM");
+    
+    XEventMonitor::instance()->start();
+}
+
+void MainWindow::RRScreenChangeEvent()
+{
+    XRRScreenResources *screen;
+    screen = XRRGetScreenResources(QX11Info::display(), QX11Info::appRootWindow());
+    XRROutputInfo *info;
+    int count = 0;
+
+    for (int i = 0; i < screen->noutput; i++) {
+        info = XRRGetOutputInfo(QX11Info::display(), screen, screen->outputs[i]);
+        if (info->connection == RR_Connected) {
+            count++;
+        }
+        XRRFreeOutputInfo(info);
+    }
+
+    onScreenCountChanged(count);
+    XRRFreeScreenResources(screen);
+}
+
+bool MainWindow::nativeEventFilter(const QByteArray &eventType, void *message, long *result)
+{
+    Q_UNUSED(result);
+    if (qstrcmp(eventType, "xcb_generic_event_t") != 0) {
+        return false;
+    }
+    xcb_generic_event_t *event = reinterpret_cast<xcb_generic_event_t*>(message);
+    const uint8_t responseType = event->response_type & ~0x80;
+    if(responseType == rr_event_base + RRScreenChangeNotify){
+        RRScreenChangeEvent();
+    }
+    return false;
+}
+
 /**
  * 有屏幕分辨率发生改变,移动Greeter窗口位置
  */
 void MainWindow::onScreenResized()
 {
-    int totalWidth = 0;
-    int totalHeight = 0;
-    for(auto screen : QGuiApplication::screens())
-    {
-        totalWidth += screen->geometry().width();
-        totalHeight += screen->geometry().height();
-    }
-    setGeometry(0, 0, totalWidth, totalHeight);
+    hide();
+    QDesktopWidget *desktop = QApplication::desktop();
+    setGeometry(desktop->geometry());
+
     qDebug() << "screen resize to " << geometry();
 
     moveToScreen(QApplication::primaryScreen());
+    show();
+    m_greeterWnd->setRootWindow();
 }
 
 void MainWindow::screenCountEvent()
 {
-    int totalWidth = 0;
-    int totalHeight = 0;
-    for(auto screen : QGuiApplication::screens())
-    {
-        totalWidth += screen->geometry().width();
-        totalHeight += screen->geometry().height();
-    }
-    setGeometry(0, 0, totalWidth, totalHeight);
+	
+    QDesktopWidget *desktop = QApplication::desktop();
+    setGeometry(desktop->geometry());
+
     moveToScreen(QApplication::primaryScreen());
     //需要重新绘制，否则背景图片大小会不正确
     repaint();
@@ -226,8 +291,14 @@ void MainWindow::screenCountEvent()
  */
 void MainWindow::onScreenCountChanged(int newCount)
 {
+    if(newCount == m_monitorCount)
+        return;
+    
+    m_monitorCount = newCount;
+
     if(newCount < 2) {
         QProcess enableMonitors;
+        //默认设置显示最大分辨率
         enableMonitors.start("xrandr --auto");
         enableMonitors.waitForFinished(-1);
     } else {
@@ -235,10 +306,6 @@ void MainWindow::onScreenCountChanged(int newCount)
         int mode = m_configuration->getValue("display-mode").toInt();
         displayService.switchDisplayMode((DisplayMode)mode);
     }
-//    if(m_first){
-//        show();
-//        activateWindow();
-//    }
 
     //在调用xrandr打开显示器以后，不能马上设置窗口大小，会设置不正确的
     //分辨率，延时500ms正常。
@@ -254,13 +321,14 @@ void MainWindow::moveToScreen(QScreen *screen)
     QRect activeScreenRect = m_activeScreen->geometry();
 
     qDebug() << "moveToScreen activeScreenRect " << activeScreenRect;
-    if(m_monitorWatcher->getMonitorCount() == 1)
+    if(QApplication::desktop()->screenCount() == 1)
         m_greeterWnd->setGeometry(QApplication::primaryScreen()->geometry());
+    else
+        m_greeterWnd->setGeometry(activeScreenRect);
 
-    m_greeterWnd->setGeometry(activeScreenRect);
-    Q_EMIT activeScreenChanged(activeScreenRect);
-
-    repaint();
+   // Q_EMIT activeScreenChanged(activeScreenRect);
+ 
+    update();
 }
 
 void MainWindow::setBackground(QSharedPointer<Background> &background)
@@ -286,15 +354,21 @@ void MainWindow::setBackground(QSharedPointer<Background> &background)
 
     //如果是第一次绘制背景，则不需要渐变
     //现在由于绘制模糊背景，不需要渐变了
-    if(!m_background.isNull())
+    if(!m_background.isNull()){
         startTransition(m_background, background);
+    }
 
     m_background = background;
+
 }
 
 void MainWindow::startTransition(QSharedPointer<Background> &from,
                                  QSharedPointer<Background> &to)
 {
+    if(!m_timer){
+         m_timer = new QTimer(this);
+         connect(m_timer, &QTimer::timeout, this, &MainWindow::onTransition);
+    }
     stopTransition();
 
     m_transition.from = from;
@@ -338,7 +412,7 @@ void MainWindow::drawBackground(QSharedPointer<Background> &background,
         return;
 
     QPainter painter(this);
-   painter.setOpacity(alpha);
+    painter.setOpacity(alpha);
 
     switch(background->type) {
     case BACKGROUND_IMAGE:
@@ -377,6 +451,13 @@ QPixmap * MainWindow::getBackground(const QString &path, const QRect &rect)
     QString resolution = QString("%1x%2").arg(rect.width()).arg(rect.height());
     QPair<QString, QString> key(path, resolution);
 
+    if(m_backgrounds.isEmpty()&&future.isRunning()){
+            future.waitForFinished();
+    }else{
+        if(!future.isFinished() && future.isStarted()){
+            future.waitForFinished();
+        }
+    }
     if(!m_backgrounds.contains(key)){
         QPixmap *pixmap =  new QPixmap(scaledPixmap(width(), height(), path));
         m_backgrounds[key] = blurPixmap(pixmap);
